@@ -20,6 +20,7 @@ build a Genetic Program Tree, and the functions to evaluate it.
 This module support both strongly and loosely typed GP.
 """
 import copy
+import math
 import random
 import re
 import sys
@@ -29,7 +30,8 @@ from collections import defaultdict, deque
 from functools import partial, wraps
 from inspect import isclass
 from operator import eq, lt
-import collections
+
+import tools        # Needed by HARM-GP
 
 ######################################
 # GP Data structure                  #
@@ -38,8 +40,9 @@ import collections
 # Define the name of type for any types.
 __type__ = object
 
+
 class PrimitiveTree(list):
-    """Tree spefically formated for optimization of genetic
+    """Tree specifically formatted for optimization of genetic
     programming operations. The tree is represented with a
     list where the nodes are appended in a depth-first order.
     The nodes appended to the tree are required to
@@ -60,22 +63,22 @@ class PrimitiveTree(list):
         if isinstance(key, slice):
             if key.start >= len(self):
                 raise IndexError("Invalid slice object (try to assign a %s"
-                    " in a tree of size %d). Even if this is allowed by the"
-                    " list object slice setter, this should not be done in"
-                    " the PrimitiveTree context, as this may lead to an"
-                    " unpredictable behavior for searchSubtree or evaluate."
-                     % (key, len(self)))
+                                 " in a tree of size %d). Even if this is allowed by the"
+                                 " list object slice setter, this should not be done in"
+                                 " the PrimitiveTree context, as this may lead to an"
+                                 " unpredictable behavior for searchSubtree or evaluate."
+                                 % (key, len(self)))
             total = val[0].arity
             for node in val[1:]:
                 total += node.arity - 1
             if total != 0:
                 raise ValueError("Invalid slice assignation : insertion of"
-                    " an incomplete subtree is not allowed in PrimitiveTree."
-                    " A tree is defined as incomplete when some nodes cannot"
-                    " be mapped to any position in the tree, considering the"
-                    " primitives' arity. For instance, the tree [sub, 4, 5,"
-                    " 6] is incomplete if the arity of sub is 2, because it"
-                    " would produce an orphan node (the 6).")
+                                 " an incomplete subtree is not allowed in PrimitiveTree."
+                                 " A tree is defined as incomplete when some nodes cannot"
+                                 " be mapped to any position in the tree, considering the"
+                                 " primitives' arity. For instance, the tree [sub, 4, 5,"
+                                 " 6] is incomplete if the arity of sub is 2, because it"
+                                 " would produce an orphan node (the 6).")
         elif val.arity != self[key].arity:
             raise ValueError("Invalid node replacement with a node of a"
                              " different arity.")
@@ -156,7 +159,7 @@ class PrimitiveTree(list):
         for elem in self:
             depth = stack.pop()
             max_depth = max(max_depth, depth)
-            stack.extend([depth+1] * elem.arity)
+            stack.extend([depth + 1] * elem.arity)
         return max_depth
 
     @property
@@ -187,22 +190,32 @@ class Primitive(object):
         'mul(1, 2)'
     """
     __slots__ = ('name', 'arity', 'args', 'ret', 'seq')
+
     def __init__(self, name, args, ret):
         self.name = name
         self.arity = len(args)
         self.args = args
         self.ret = ret
-        args = ", ".join(map("{{{0}}}".format, list(range(self.arity))))
+        args = ", ".join(map("{{{0}}}".format, range(self.arity)))
         self.seq = "{name}({args})".format(name=self.name, args=args)
 
     def format(self, *args):
         return self.seq.format(*args)
 
+    def __eq__(self, other):
+        if type(self) is type(other):
+            return all(getattr(self, slot) == getattr(other, slot)
+                       for slot in self.__slots__)
+        else:
+            return NotImplemented
+
+
 class Terminal(object):
     """Class that encapsulates terminal primitive in expression. Terminals can
     be values or 0-arity functions.
     """
     __slots__ = ('name', 'value', 'ret', 'conv_fct')
+
     def __init__(self, terminal, symbolic, ret):
         self.ret = ret
         self.value = terminal
@@ -216,6 +229,14 @@ class Terminal(object):
     def format(self):
         return self.conv_fct(self.value)
 
+    def __eq__(self, other):
+        if type(self) is type(other):
+            return all(getattr(self, slot) == getattr(other, slot)
+                       for slot in self.__slots__)
+        else:
+            return NotImplemented
+
+
 class Ephemeral(Terminal):
     """Class that encapsulates a terminal which value is set when the
     object is created. To mutate the value, a new object has to be
@@ -231,6 +252,7 @@ class Ephemeral(Terminal):
         """
         raise NotImplementedError
 
+
 class PrimitiveSetTyped(object):
     """Class that contains the primitives that can be used to solve a
     Strongly Typed GP problem. The set also defined the researched
@@ -243,7 +265,7 @@ class PrimitiveSetTyped(object):
         # setting "__builtins__" to None avoid the context
         # being polluted by builtins function when evaluating
         # GP expression.
-        self.context = {"__builtins__" : None}
+        self.context = {"__builtins__": None}
         self.mapping = dict()
         self.terms_count = 0
         self.prims_count = 0
@@ -271,12 +293,12 @@ class PrimitiveSetTyped(object):
 
     def _add(self, prim):
         def addType(dict_, ret_type):
-            if not ret_type in dict_:
+            if ret_type not in dict_:
                 new_list = []
-                for type_, list_ in list(dict_.items()):
+                for type_, list_ in dict_.items():
                     if issubclass(type_, ret_type):
                         for item in list_:
-                            if not item in new_list:
+                            if item not in new_list:
                                 new_list.append(item)
                 dict_[ret_type] = new_list
 
@@ -310,10 +332,10 @@ class PrimitiveSetTyped(object):
         prim = Primitive(name, in_types, ret_type)
 
         assert name not in self.context or \
-               self.context[name] is primitive, \
-               "Primitives are required to have a unique name. " \
-               "Consider using the argument 'name' to rename your "\
-               "second '%s' primitive." % (name,)
+            self.context[name] is primitive, \
+            "Primitives are required to have a unique name. " \
+            "Consider using the argument 'name' to rename your "\
+            "second '%s' primitive." % (name,)
 
         self._add(prim)
         self.context[prim.name] = primitive
@@ -333,13 +355,13 @@ class PrimitiveSetTyped(object):
         :param name: defines the name of the terminal in the expression.
         """
         symbolic = False
-        if name is None and isinstance(terminal, collections.Callable):
+        if name is None and callable(terminal):
             name = terminal.__name__
 
         assert name not in self.context, \
-               "Terminals are required to have a unique name. " \
-               "Consider using the argument 'name' to rename your "\
-               "second %s terminal." % (name,)
+            "Terminals are required to have a unique name. " \
+            "Consider using the argument 'name' to rename your "\
+            "second %s terminal." % (name,)
 
         if name is not None:
             self.context[name] = terminal
@@ -364,9 +386,9 @@ class PrimitiveSetTyped(object):
         :param ret_type: type of the object returned by *ephemeral*.
         """
         module_gp = globals()
-        if not name in module_gp:
-            class_ = type(name, (Ephemeral,), {'func' : staticmethod(ephemeral),
-                                               'ret' : ret_type})
+        if name not in module_gp:
+            class_ = type(name, (Ephemeral,), {'func': staticmethod(ephemeral),
+                                               'ret': ret_type})
             module_gp[name] = class_
         else:
             class_ = module_gp[name]
@@ -380,7 +402,7 @@ class PrimitiveSetTyped(object):
             else:
                 raise Exception("Ephemerals should be named differently "
                                 "than classes defined in the gp module.")
-        
+
         self._add(class_)
         self.terms_count += 1
 
@@ -401,12 +423,13 @@ class PrimitiveSetTyped(object):
         """
         return self.terms_count / float(self.terms_count + self.prims_count)
 
+
 class PrimitiveSet(PrimitiveSetTyped):
     """Class same as :class:`~deap.gp.PrimitiveSetTyped`, except there is no
     definition of type.
     """
     def __init__(self, name, arity, prefix="ARG"):
-        args = [__type__]*arity
+        args = [__type__] * arity
         PrimitiveSetTyped.__init__(self, name, args, __type__, prefix)
 
     def addPrimitive(self, primitive, arity, name=None):
@@ -451,11 +474,12 @@ def compile(expr, pset):
         return eval(code, pset.context, {})
     except MemoryError:
         _, _, traceback = sys.exc_info()
-        raise MemoryError("DEAP : Error in tree evaluation :"
-        " Python cannot evaluate a tree higher than 90. "
-        "To avoid this problem, you should use bloat control on your "
-        "operators. See the DEAP documentation for more information. "
-        "DEAP will now abort.").with_traceback(traceback)
+        raise MemoryError, ("DEAP : Error in tree evaluation :"
+                            " Python cannot evaluate a tree higher than 90. "
+                            "To avoid this problem, you should use bloat control on your "
+                            "operators. See the DEAP documentation for more information. "
+                            "DEAP will now abort."), traceback
+
 
 def compileADF(expr, psets):
     """Compile the expression represented by a list of trees. The first
@@ -476,12 +500,13 @@ def compileADF(expr, psets):
     """
     adfdict = {}
     func = None
-    for pset, subexpr in reversed(list(zip(psets, expr))):
+    for pset, subexpr in reversed(zip(psets, expr)):
         pset.context.update(adfdict)
         func = compile(subexpr, pset)
-        adfdict.update({pset.name : func})
+        adfdict.update({pset.name: func})
     return func
 
+
 ######################################
 # GP Program generation functions    #
 ######################################
@@ -493,7 +518,8 @@ def genFull(pset, min_, max_, type_=None):
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: A full tree with all leaves at the same depth.
     """
     def condition(height, depth):
@@ -501,6 +527,7 @@ def genFull(pset, min_, max_, type_=None):
         return depth == height
     return generate(pset, min_, max_, condition, type_)
 
+
 def genGrow(pset, min_, max_, type_=None):
     """Generate an expression where each leaf might have a different depth
     between *min* and *max*.
@@ -509,7 +536,8 @@ def genGrow(pset, min_, max_, type_=None):
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: A grown tree with leaves at possibly different depths.
     """
     def condition(height, depth):
@@ -517,9 +545,10 @@ def genGrow(pset, min_, max_, type_=None):
         or when it is randomly determined that a a node should be a terminal.
         """
         return depth == height or \
-               (depth >= min_ and random.random() < pset.terminalRatio)
+            (depth >= min_ and random.random() < pset.terminalRatio)
     return generate(pset, min_, max_, condition, type_)
 
+
 def genHalfAndHalf(pset, min_, max_, type_=None):
     """Generate an expression with a PrimitiveSet *pset*.
     Half the time, the expression is generated with :func:`~deap.gp.genGrow`,
@@ -529,12 +558,14 @@ def genHalfAndHalf(pset, min_, max_, type_=None):
     :param min_: Minimum height of the produced trees.
     :param max_: Maximum Height of the produced trees.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: Either, a full or a grown tree.
     """
     method = random.choice((genGrow, genFull))
     return method(pset, min_, max_, type_)
 
+
 def genRamped(pset, min_, max_, type_=None):
     """
     .. deprecated:: 1.0
@@ -544,6 +575,7 @@ def genRamped(pset, min_, max_, type_=None):
                   FutureWarning)
     return genHalfAndHalf(pset, min_, max_, type_)
 
+
 def generate(pset, min_, max_, condition, type_=None):
     """Generate a Tree as a list of list. The tree is build
     from the root to the leaves, and it stop growing when the
@@ -556,7 +588,8 @@ def generate(pset, min_, max_, condition, type_=None):
                       the height of the tree to build and the current
                       depth in the tree.
     :param type_: The type that should return the tree when called, when
-                  :obj:`None` (default) no return type is enforced.
+                  :obj:`None` (default) the type of :pset: (pset.ret)
+                  is assumed.
     :returns: A grown tree with leaves at possibly different depths
               dependending on the condition function.
     """
@@ -572,9 +605,9 @@ def generate(pset, min_, max_, condition, type_=None):
                 term = random.choice(pset.terminals[type_])
             except IndexError:
                 _, _, traceback = sys.exc_info()
-                raise IndexError("The gp.generate function tried to add "\
+                raise IndexError, "The gp.generate function tried to add "\
                                   "a terminal of type '%s', but there is "\
-                                  "none available." % (type_,)).with_traceback(traceback)
+                                  "none available." % (type_,), traceback
             if isclass(term):
                 term = term()
             expr.append(term)
@@ -583,12 +616,12 @@ def generate(pset, min_, max_, condition, type_=None):
                 prim = random.choice(pset.primitives[type_])
             except IndexError:
                 _, _, traceback = sys.exc_info()
-                raise IndexError("The gp.generate function tried to add "\
+                raise IndexError, "The gp.generate function tried to add "\
                                   "a primitive of type '%s', but there is "\
-                                  "none available." % (type_,)).with_traceback(traceback)
+                                  "none available." % (type_,), traceback
             expr.append(prim)
             for arg in reversed(prim.args):
-                stack.append((depth+1, arg))
+                stack.append((depth + 1, arg))
     return expr
 
 
@@ -613,8 +646,8 @@ def cxOnePoint(ind1, ind2):
     types2 = defaultdict(list)
     if ind1.root.ret == __type__:
         # Not STGP optimization
-        types1[__type__] = range(1, len(ind1))
-        types2[__type__] = range(1, len(ind2))
+        types1[__type__] = xrange(1, len(ind1))
+        types2[__type__] = xrange(1, len(ind2))
         common_types = [__type__]
     else:
         for idx, node in enumerate(ind1[1:], 1):
@@ -725,7 +758,7 @@ def mutNodeReplacement(individual, pset):
     index = random.randrange(1, len(individual))
     node = individual[index]
 
-    if node.arity == 0: # Terminal
+    if node.arity == 0:  # Terminal
         term = random.choice(pset.terminals[node.ret])
         if isclass(term):
             term = term()
@@ -736,6 +769,7 @@ def mutNodeReplacement(individual, pset):
 
     return individual,
 
+
 def mutEphemeral(individual, mode):
     """This operator works on the constants of the tree *individual*. In
     *mode* ``"one"``, it will change the value of one of the individual
@@ -763,6 +797,7 @@ def mutEphemeral(individual, mode):
 
     return individual,
 
+
 def mutInsert(individual, pset):
     """Inserts a new branch at a random position in *individual*. The subtree
     at the chosen position is used as child node of the created subtree, in
@@ -797,11 +832,12 @@ def mutInsert(individual, pset):
                 term = term()
             new_subtree[i] = term
 
-    new_subtree[position:position+1] = individual[slice_]
+    new_subtree[position:position + 1] = individual[slice_]
     new_subtree.insert(0, new_node)
     individual[slice_] = new_subtree
     return individual,
 
+
 def mutShrink(individual):
     """This operator shrinks the *individual* by chosing randomly a branch and
     replacing it with one of the branch's arguments (also randomly chosen).
@@ -821,8 +857,8 @@ def mutShrink(individual):
     if len(iprims) != 0:
         index, prim = random.choice(iprims)
         arg_idx = random.choice([i for i, type_ in enumerate(prim.args) if type_ == prim.ret])
-        rindex = index+1
-        for _ in range(arg_idx+1):
+        rindex = index + 1
+        for _ in range(arg_idx + 1):
             rslice = individual.searchSubtree(rindex)
             subtree = individual[rslice]
             rindex += len(subtree)
@@ -836,6 +872,7 @@ def mutShrink(individual):
 # GP bloat control decorators        #
 ######################################
 
+
 def staticLimit(key, max_value):
     """Implement a static limit on some measurement on a GP tree, as defined
     by Koza in [Koza1989]. It may be used to decorate both crossover and
@@ -847,8 +884,8 @@ def staticLimit(key, max_value):
     depth), because it can ensure that no tree higher than this limit will ever
     be accepted in the population, except if it was generated at initialization
     time.
-    
-    :param key: The function to use in order the get the wanted value. For 
+
+    :param key: The function to use in order the get the wanted value. For
                 instance, on a GP tree, ``operator.attrgetter('height')`` may
                 be used to set a depth limit, and ``len`` to set a size limit.
     :param max_value: The maximum value allowed for the given measurement.
@@ -876,6 +913,209 @@ def staticLimit(key, max_value):
         return wrapper
     return decorator
 
+
+######################################
+# GP bloat control algorithms        #
+######################################
+
+def harm(population, toolbox, cxpb, mutpb, ngen,
+         alpha, beta, gamma, rho, nbrindsmodel=-1, mincutoff=20,
+         stats=None, halloffame=None, verbose=__debug__):
+    """Implement bloat control on a GP evolution using HARM-GP, as defined in
+    [Gardner2015]. It is implemented in the form of an evolution algorithm
+    (similar to :func:`~deap.algorithms.eaSimple`).
+
+    :param population: A list of individuals.
+    :param toolbox: A :class:`~deap.base.Toolbox` that contains the evolution
+                    operators.
+    :param cxpb: The probability of mating two individuals.
+    :param mutpb: The probability of mutating an individual.
+    :param ngen: The number of generation.
+    :param alpha: The HARM *alpha* parameter.
+    :param beta: The HARM *beta* parameter.
+    :param gamma: The HARM *gamma* parameter.
+    :param rho: The HARM *rho* parameter.
+    :param nbrindsmodel: The number of individuals to generate in order to
+                            model the natural distribution. -1 is a special
+                            value which uses the equation proposed in
+                            [Gardner2015] to set the value of this parameter :
+                            max(2000, len(population))
+    :param mincutoff: The absolute minimum value for the cutoff point. It is
+                        used to ensure that HARM does not shrink the population
+                        too much at the beginning of the evolution. The default
+                        value is usually fine.
+    :param stats: A :class:`~deap.tools.Statistics` object that is updated
+                  inplace, optional.
+    :param halloffame: A :class:`~deap.tools.HallOfFame` object that will
+                       contain the best individuals, optional.
+    :param verbose: Whether or not to log the statistics.
+    :returns: The final population
+    :returns: A class:`~deap.tools.Logbook` with the statistics of the
+              evolution
+
+    This function expects the :meth:`toolbox.mate`, :meth:`toolbox.mutate`,
+    :meth:`toolbox.select` and :meth:`toolbox.evaluate` aliases to be
+    registered in the toolbox.
+
+    .. note::
+       The recommended values for the HARM-GP parameters are *alpha=0.05*,
+       *beta=10*, *gamma=0.25*, *rho=0.9*. However, these parameters can be
+       adjusted to perform better on a specific problem (see the relevant
+       paper for tuning information). The number of individuals used to
+       model the natural distribution and the minimum cutoff point are less
+       important, their default value being effective in most cases.
+
+    .. [Gardner2015] M.-A. Gardner, C. Gagne, and M. Parizeau, Controlling
+        Code Growth by Dynamically Shaping the Genotype Size Distribution,
+        Genetic Programming and Evolvable Machines, 2015,
+        DOI 10.1007/s10710-015-9242-8
+
+    """
+    def _genpop(n, pickfrom=[], acceptfunc=lambda s: True, producesizes=False):
+        # Generate a population of n individuals, using individuals in
+        # *pickfrom* if possible, with a *acceptfunc* acceptance function.
+        # If *producesizes* is true, also return a list of the produced
+        # individuals sizes.
+        # This function is used 1) to generate the natural distribution
+        # (in this case, pickfrom and acceptfunc should be let at their
+        # default values) and 2) to generate the final population, in which
+        # case pickfrom should be the natural population previously generated
+        # and acceptfunc a function implementing the HARM-GP algorithm.
+        producedpop = []
+        producedpopsizes = []
+        while len(producedpop) < n:
+            if len(pickfrom) > 0:
+                # If possible, use the already generated
+                # individuals (more efficient)
+                aspirant = pickfrom.pop()
+                if acceptfunc(len(aspirant)):
+                    producedpop.append(aspirant)
+                    if producesizes:
+                        producedpopsizes.append(len(aspirant))
+            else:
+                opRandom = random.random()
+                if opRandom < cxpb:
+                    # Crossover
+                    aspirant1, aspirant2 = toolbox.mate(*map(toolbox.clone,
+                                                             toolbox.select(population, 2)))
+                    del aspirant1.fitness.values, aspirant2.fitness.values
+                    if acceptfunc(len(aspirant1)):
+                        producedpop.append(aspirant1)
+                        if producesizes:
+                            producedpopsizes.append(len(aspirant1))
+
+                    if len(producedpop) < n and acceptfunc(len(aspirant2)):
+                        producedpop.append(aspirant2)
+                        if producesizes:
+                            producedpopsizes.append(len(aspirant2))
+                else:
+                    aspirant = toolbox.clone(toolbox.select(population, 1)[0])
+                    if opRandom - cxpb < mutpb:
+                        # Mutation
+                        aspirant = toolbox.mutate(aspirant)[0]
+                        del aspirant.fitness.values
+                    if acceptfunc(len(aspirant)):
+                        producedpop.append(aspirant)
+                        if producesizes:
+                            producedpopsizes.append(len(aspirant))
+
+        if producesizes:
+            return producedpop, producedpopsizes
+        else:
+            return producedpop
+
+    def halflifefunc(x):
+        return x * float(alpha) + beta
+
+    if nbrindsmodel == -1:
+        nbrindsmodel = max(2000, len(population))
+
+    logbook = tools.Logbook()
+    logbook.header = ['gen', 'nevals'] + (stats.fields if stats else [])
+
+    # Evaluate the individuals with an invalid fitness
+    invalid_ind = [ind for ind in population if not ind.fitness.valid]
+    fitnesses = toolbox.map(toolbox.evaluate, invalid_ind)
+    for ind, fit in zip(invalid_ind, fitnesses):
+        ind.fitness.values = fit
+
+    if halloffame is not None:
+        halloffame.update(population)
+
+    record = stats.compile(population) if stats else {}
+    logbook.record(gen=0, nevals=len(invalid_ind), **record)
+    if verbose:
+        print logbook.stream
+
+    # Begin the generational process
+    for gen in range(1, ngen + 1):
+        # Estimation population natural distribution of sizes
+        naturalpop, naturalpopsizes = _genpop(nbrindsmodel, producesizes=True)
+
+        naturalhist = [0] * (max(naturalpopsizes) + 3)
+        for indsize in naturalpopsizes:
+            # Kernel density estimation application
+            naturalhist[indsize] += 0.4
+            naturalhist[indsize - 1] += 0.2
+            naturalhist[indsize + 1] += 0.2
+            naturalhist[indsize + 2] += 0.1
+            if indsize - 2 >= 0:
+                naturalhist[indsize - 2] += 0.1
+
+        # Normalization
+        naturalhist = [val * len(population) / nbrindsmodel for val in naturalhist]
+
+        # Cutoff point selection
+        sortednatural = sorted(naturalpop, key=lambda ind: ind.fitness)
+        cutoffcandidates = sortednatural[int(len(population) * rho - 1):]
+        # Select the cutoff point, with an absolute minimum applied
+        # to avoid weird cases in the first generations
+        cutoffsize = max(mincutoff, len(min(cutoffcandidates, key=len)))
+
+        # Compute the target distribution
+        def targetfunc(x):
+            return (gamma * len(population) * math.log(2) /
+                    halflifefunc(x)) * math.exp(-math.log(2) *
+                                                (x - cutoffsize) / halflifefunc(x))
+        targethist = [naturalhist[binidx] if binidx <= cutoffsize else
+                      targetfunc(binidx) for binidx in range(len(naturalhist))]
+
+        # Compute the probabilities distribution
+        probhist = [t / n if n > 0 else t for n, t in zip(naturalhist, targethist)]
+
+        def probfunc(s):
+            return probhist[s] if s < len(probhist) else targetfunc(s)
+
+        def acceptfunc(s):
+            return random.random() <= probfunc(s)
+
+        # Generate offspring using the acceptance probabilities
+        # previously computed
+        offspring = _genpop(len(population), pickfrom=naturalpop,
+                            acceptfunc=acceptfunc, producesizes=False)
+
+        # Evaluate the individuals with an invalid fitness
+        invalid_ind = [ind for ind in offspring if not ind.fitness.valid]
+        fitnesses = toolbox.map(toolbox.evaluate, invalid_ind)
+        for ind, fit in zip(invalid_ind, fitnesses):
+            ind.fitness.values = fit
+
+        # Update the hall of fame with the generated individuals
+        if halloffame is not None:
+            halloffame.update(offspring)
+
+        # Replace the current population by the offspring
+        population[:] = offspring
+
+        # Append the current generation statistics to the logbook
+        record = stats.compile(population) if stats else {}
+        logbook.record(gen=gen, nevals=len(invalid_ind), **record)
+        if verbose:
+            print logbook.stream
+
+    return population, logbook
+
+
 def graph(expr):
     """Construct the graph of a tree expression. The tree expression must be
     valid. It returns in order a node list, an edge list, and a dictionary of
@@ -932,7 +1172,7 @@ def graph(expr):
        <http://networkx.lanl.gov/pygraphviz/>`_ as the nodes might be plotted
        out of order when using `NetworX <http://networkx.github.com/>`_.
     """
-    nodes = list(range(len(expr)))
+    nodes = range(len(expr))
     edges = list()
     labels = dict()
 
